@@ -19,7 +19,7 @@ In the copy-local protocol, the frontend can terminate the cycle by sending a
 CopyError message, which will cause the COPY SQL statement to fail with an error.
 """
 
-from __future__ import print_function, division, absolute_import
+from __future__ import print_function, division, absolute_import, annotations
 
 from struct import pack
 
